// Copyright IBM Corp. 2013,2017. All Rights Reserved.
// Node module: strong-remoting
// This file is licensed under the Artistic License 2.0.
// License text available at https://opensource.org/licenses/Artistic-2.0

/* Copyright (c) 2013 StrongLoop, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

'use strict';

const extend = require('util')._extend;

exports.createSharedClass = function createSharedClass(config) {
  // create a class that can be remoted
  const SharedClass = function(id) {
    this.id = id;
  };
  extend(SharedClass, config);

  SharedClass.shared = true;

  SharedClass.sharedCtor = function(id, cb) {
    // allow tests to override the implementation of shared ctor
    // while preserving remoting metadata
    this._sharedCtor(id, cb);
  };

  extend(SharedClass.sharedCtor, {
    shared: true,
    accepts: [{arg: 'id', type: 'any', http: {source: 'path'}}],
    http: {path: '/:id'},
    returns: {arg: 'instance', type: 'object', root: true},
  });

  SharedClass._sharedCtor = function(id, cb) {
    cb(null, new SharedClass(id));
  };

  return SharedClass;
};
